#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
ctrl_acq.py :
    manages acquisition
"""
# standard library imports
import os.path

# related third party imports
import gst
from pitivi.signalgroup import SignalGroup
from pitivi.action import ViewAction
from pitivi.pipeline import Pipeline, PipelineError

# local application/library specific imports
from luciole.media.lgst.webcam_factory import WebcamSourceFactory
import luciole.constants as LCONST
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.media.image as LI

from luciole.ctrl.base import ProjectMode

class ProjectModeAcquisition(ProjectMode) :
    """ 
    class in charge of Acquisition.
    High level class 
    """
    MODES = (   "INVALID",
                "NO_ACQUISITION",
                "ACQUISITION",
                "ACQUISITION_WITH_MIXER",
                )

    __signals__ = {
                    'acquirer-ready' : [],
                    'acquirer-error' : ['msg'],
                    'acquisition-started' : ['with_mixer'],
                    'acquisition-stopped' : [],
                    'snapshot-on-progress' : [],
                    'snapshot-taken' : ['rush_image'],
                }

    def __init__(self, ctrl_project) :
        self._mode = "INVALID"
        ProjectMode.__init__(self, ctrl_project, "ACQUISITION") 
        self.gui_signals = SignalGroup()
    
        self._source_factory = None
        self._view_action = None
    
    def prepare(self, project) :
        """ prepare acquistion """
        ProjectMode.prepare(self, project)
        # prepare acqusisition
        _project = project.props
        if _project != None :
            if _project['hardtype'] == LCONST.WEBCAM :
                # create webcam source
                _capture_path = os.path.join(
                        _project['project_dir'], 
                        LCONST.ACQUIRED_IMAGE_NAME)
                self.debug ("Image capture path : %s", _capture_path)
                self._source_factory = \
                        WebcamSourceFactory(capture_path = _capture_path,
                                            with_mixer = False)

                self.debug('webcam filter : %s', 
                        self.project.props['webcam_data']) 
                self._set_webcam_params(
                        self.project.props['webcam_data'],
                        self._source_factory)
                
                # set video resize/scale mode (or not)
                if _project['resize'] == 'yes' :
                    self._source_factory.with_scale = True
                else :
                    self._source_factory.with_scale = False

                self._connect_to_factory(self._source_factory, _capture_path )
                
                # create pipeline and resource
                self._pipeline = Pipeline()
                self._view_action = ViewAction()
                self._view_action.addProducers(self._source_factory)
                
                self._mode = "NO_ACQUISITION"
                self.debug(
                    'emit acquirer-ready, Webcam factory : %s, pipeline : %s',
                    self._source_factory, self._pipeline )
                self.emit('acquirer-ready')


    def release(self) :
        """ release ctrl_acq used objects 
        """
        ProjectMode.release(self)
        self.debug('releasing acquisition')
        self._disconnect_from_factory(self._source_factory)
        self._mode = "INVALID"
        self._pipeline.release()
        self._pipeline = None
        self._source_factory = None
        self._view_action = None

    def active_viewer(self) :
        """ set gui viewer with acquisition view """
        if self._viewer :
            self._viewer.setPipeline(None)
            self._viewer.hideSlider()
            self.debug('active viewer - Pipeline : %s, action %s', 
                self._pipeline,
                self._view_action)
            self._viewer.setAction(self._view_action)
            self._viewer.setPipeline(self._pipeline, "ACQUISITION")
        else :
            self.error("No viewer set")
        

    def start(self, with_mixer = False) :
        """ start acquisition """
        # check if image view mode
        # in that case restart acquisition mode
        if self.ctrl_project.mode == 'IMAGE_VIEW' :
            self.ctrl_project.acquistion_mode()
            return

        if self._mode == "NO_ACQUISITION" :
            if with_mixer == True :
                pass
            # TODO :  deactivated code mixer does not work
                #self._source_factory.active_mixer(True)
                #self._mode = "ACQUISITION_WITH_MIXER"
            else :
                #self._source_factory.active_mixer(False)
                self._mode = "ACQUISITION"
            try :     
                self._pipeline.play()
            except PipelineError :
                self.warning('Fail to start pipeline')
                self._mode = "NO_ACQUISITION"
            else :
                self.debug('send acquisition-started - mode : %s', self._mode)
                self.emit('acquisition-started', with_mixer)


        else :
            self.error("Cannot start acquisition."
                        "Controller in incorrect mode (%s)", 
            self._mode)


    def stop(self) :
        """ stop acquisition """
        self.debug('stop acquisition - mode  %s', self._mode)
        if self._mode in ("ACQUISITION", "ACQUISITION_WITH_MIXER") :
            if self._mode == "ACQUISITION_WITH_MIXER" :
                pass
                #self.acquirer.deactive_onion_skin()
            self._pipeline.stop()
            self._mode = "NO_ACQUISITION"
            self.debug('emit acquisition-stopped')
            self.emit('acquisition-stopped')
        else :
            self.error(
            "Cannot stop acquistion. Controller in incorrect mode (%s)", 
            self._mode)
    

    def do_snapshot(self) :
        """ make/request image snapshot """
        if self._mode in ("ACQUISITION", "ACQUISITION_WITH_MIXER") :
            self._source_factory.capture()

        else :
            self.error(
            "Cannot take snapshot. Controller in incorrect mode (%s)", 
            self._mode)

    def active_mixer(self, is_active) :
        """ active the mixer (onion skin)"""

        # check with current mixer status 
        self.debug('active_mixer : %s', is_active)
        if self.project.props['is-mixer-active'] == is_active :
            self.info('No mixer status to change')
            return
        
        

        # stop pipeline,  deactivate producer and remove old webcam factory 
        self._pipeline.stop()
        self._view_action.deactivate()
        self._view_action.removeProducers(self._source_factory)
        
        # create new webcam factory
        _capture_path = os.path.join(
                        self.project.props['project_dir'], 
                        LCONST.ACQUIRED_IMAGE_NAME)
        self.debug ("Image capture path : %s", _capture_path)

        self._source_factory =  WebcamSourceFactory(
                                capture_path = _capture_path, 
                                with_mixer = is_active,
                                image2mix = self.project.props['image2mix'] )
        # set video resize/scale mode (or not)
        if self.project.props['resize'] == 'yes' :
            self._source_factory.with_scale = True
        else :
            self._source_factory.with_scale = False


        # add producer 
        self._view_action.addProducers(self._source_factory)
        # activate it --> regeneration of pipeline bins
        self._view_action.activate()
        # set image alpha if webcam active 
        if is_active == True :
            self._source_factory.alpha_value = self.project.props['alpha']
        
            self.debug(' image to mix is %s with alpha = %s',
                self._source_factory.image2mix,
                self._source_factory.alpha_value)

        # start acquisition
        self._pipeline.play()
        
        self.project.props['is-mixer-active'] = is_active
        
    def _connect_to_factory(self, factory, _capture_path) :
        """ connect to acqusistion factory signals """
        factory.connect("image-capture-done",
                        self._image_capture_done_cb,
                        _capture_path )
    def _disconnect_from_factory(self, factory) :
        """ disconnect signals from factory """
        factory.disconnect_by_function(self._image_capture_done_cb)

    def _image_capture_done_cb( self, factory, _capture_path) :
        """ image capture done callback, 
            signal received from factory """

        self.debug('RX image-capture-done')
        _rush_image = self._process_snapshot(_capture_path)
        self.debug('emit snapshot-taken : %s', _rush_image.name)
        self.emit('snapshot-taken', _rush_image)

    def _process_snapshot(self, capture_path) :
        """ 
        process image capture to insert in the luciole project 
        """
         # get acquired image name
        _acq_image = capture_path

        # build temp impage path
        _temp_dir = os.path.join(self.project.props['project_dir'], 'tmp')
        # copy name
        _ac_image_temp = os.path.join(_temp_dir, _acq_image)
        # resized copy name
        _ac_image_temp_rz = \
            os.path.join(_temp_dir, LCONST.ACQUIRED_IMAGE_NAME_RZ)

        # build rush image name
        _basename = LCONST.RUSH_FILENAME_TPL % self.project.rush.rush_index
        _rush_image = \
            os.path.join(self.project.props['project_dir'],
                        self.project.props['rush_dir'],
                        _basename)

        try :  
            # 1. move image acquired image to tmp dir
            LT.movef(_acq_image, _ac_image_temp)
            # 2. resize image result is in _ac_image_temp_rz
            if self.project.props['resize'] == 'yes' :
                _rz_obj = LI.ImageResize(_ac_image_temp, _ac_image_temp_rz )
                _rz_obj.convert()
            else :
                _ac_image_temp_rz = _ac_image_temp
            
            # 3. move resized image to rush dire 
            LT.movef(_ac_image_temp_rz, _rush_image)
        
        except LEXCEP.LucioException, _err_msg :
            self.error(_err_msg)
        else :
            self.debug("create rush image : %s", _basename) 
            # 4. append image to rush list
            _rush_image = self.project.rush.append(_basename)
            
            # always update the image 2 mix, even if mixer is not active
            # used to memorize the last capture 
            #self.acquirer.Image2Mix = _rush_image.path
            
            # project modified
            self.project.props['rush_images'].append(_basename)
            self.project.props['capture_images'].append(_basename)

            self.ctrl_project.is_modified = True
            
            self.debug("capture post-treatemnt done ") 
            return _rush_image

    def _set_webcam_params(self, webcam_props, factory) :
        """ 
        set webcam parameters according project propperties
        """
        #retrieve params value
        _wcam_params = {
            'wcam_driver': {
                'element': webcam_props['v4ldriver'] ,
                'properties' : {
                    'device':webcam_props['device-file'],
                    },
                },
            
            'wcam_filter' : {
                'media_type' :gst.Caps(webcam_props['caps']),
                },
            }

        # set params on factory
        factory.webcam_params =  _wcam_params
        self.debug(" webcam params set : %s", factory.webcam_params)
        

        




