#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
ctrl.py :
   Main application controller
"""
# standard library imports
import os
import os.path
import locale
import gettext
_ = gettext.gettext

# related third party imports
import gobject
from pitivi.log import log

from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable

# local application/library specific imports
import luciole.base.options as OPTIONS
import luciole.conf as LCONF
import luciole.constants as LCONST
import luciole.ctrl.ctrl_project as CTRL_PROJECT
import luciole.gui.main_window as GUI_MAIN
import luciole.gui.gui_ctrl as GUI_CTRL
import luciole.media.lgst.play_sound as LSOUND
#####



class CtrlAppGui(Signallable, Loggable) :
    """ Interface between gui and App"""
           
    def __init__(self, app) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.app = app
        self.project_ctrl = self.app.project_ctrl
        self.gui = self.app.gui
        self._connect_gui_and_project()

    def _connect_gui_and_project(self) :
        """ conect gui signals to project """ 
        # application high level actions 
        self.gui.gui_actions.connect('open-project', self._open_project_cb)
        self.gui.gui_actions.connect('new-project', self._new_project_cb)
        self.gui.gui_actions.connect('quit-app', self._quit_cb) 
        
        # Project level actions  
        self.gui.gui_actions.connect('save-project',
                self._save_project_cb) 
        self.gui.gui_actions.connect('save-as-project', 
                self._save_as_project_cb) 
        self.gui.gui_actions.connect('close-project',
                self._close_project_cb) 
        self.gui.gui_actions.connect('acquisition-mode',
                self._acq_mode_cb) 
        self.gui.gui_actions.connect('timeline-mode',
                self._timeline_mode_cb) 
        self.gui.gui_actions.connect('import-sound',
                self._import_sound_cb) 
        self.gui.gui_actions.connect('image-view',
                self._image_view_cb) 
        self.gui.gui_actions.connect('import-images',
                self._import_images_cb) 
        self.gui.gui_actions.connect('show-project-properties',
                self._show_project_properties_cb) 


        # Acqusition actions
        self.gui.gui_actions.connect('start-acquistion', 
                self._start_acq_cb) 
        self.gui.gui_actions.connect('stop-acquistion',
                self._stop_acq_cb) 
        self.gui.gui_actions.connect('take-snapshot',
                self._take_snapshot_cb)
        self.gui.gui_actions.connect('active-mixer',
                self._active_mixer_cb)
        
        # Timeline actions
        self.gui.gui_actions.connect('change-timeline-order',
                self._change_timeline_order_cb) 
        self.gui.gui_actions.connect('insert-image-on-timeline',
                self._insert_image_on_timeline_cb) 
        self.gui.gui_actions.connect('remove-image-on-timeline',
                self._remove_image_on_timeline_cb) 
        self.gui.gui_actions.connect('active-sound',
                self._active_sound_cb) 
        self.gui.gui_actions.connect('deactive-sound',
                self._deactive_sound_cb) 
        self.gui.gui_actions.connect('sound-stop-with-video',
                self._sound_stop_with_video_cb) 
        self.gui.gui_actions.connect('change-framerate', 
                self._change_framerate_cb) 
        

    #
    #  application high level callbacks 
    #
    
    def _open_project_cb(self, gui, path) :
        """ GUI open project callback"""
        self.debug('RX open-project signal')
        self.project_ctrl.open(path)
    
    def _new_project_cb(self, gui ) :
        """ GUI new project callback"""
        self.debug('RX new-project signal' )
        self.project_ctrl.new_project()


    def _quit_cb(self, gui): 
        """ GUI quit callback"""
        self.app.quit()
        return True
    #
    # Project level callbacks 
    #
    def _save_project_cb(self, gui) :
        """ GUI save project callback"""
        self.debug('RX save-project signal')
        self.project_ctrl.save()

    def _save_as_project_cb(self, gui, path) :
        """ GUI save as project callback"""
        self.debug('RX save-as-project signal')
        self.project_ctrl.save(path)


    def _close_project_cb(self, gui) :
        """ GUI close project callback"""
        self.debug('RX close-project signal')
        self.project_ctrl.close()

        
    def _acq_mode_cb(self, gui) :
        """ GUI acquisition mode callback"""
        self.debug('RX acquisition-mode signal')
        self.project_ctrl.acquisition_mode()

    def _timeline_mode_cb(self, gui) :
        """ GUI timeline mode callback"""
        self.debug('RX timeline-mode signal')
        self.project_ctrl.timeline_mode()
    
    def _import_sound_cb(self, gui, path) :
        """ GUI import sound callback"""
        self.debug('RX import-sound signal : %s', path)
        self.project_ctrl.ctrl_tmln.add_sound_track(path)
    
    def _image_view_cb(self, gui, origin, image_name, position = 0) :
        """ GUI image view callback"""
        self.debug('RX image-view signal : origin %s image %s',
                origin, 
                image_name)
        self.project_ctrl.img_vw_mode(origin, image_name, position)
    
    def _import_images_cb(self, gui, paths) :
        """ GUI import images callback"""
        self.debug('RX import-images signal : %s', paths  )
        self.project_ctrl.import_images(paths)
    
    def _show_project_properties_cb(self, gui) :
        """ GUI show project properties callback """
        self.debug('RX show-project-properties' )
        self.project_ctrl.show_project_properties()



    #
    # Acqusition callbacks
    # 

    def _start_acq_cb(self, gui) :
        """ GUI start acqusisition callback"""
        self.debug(' RX start-acquistion signal')
        self.project_ctrl.ctrl_acq.start()
    
    def _stop_acq_cb(self, gui) :
        """ GUI stop acqusisition callback"""
        self.debug(' RX stop-acquistion signal')
        self.project_ctrl.ctrl_acq.stop()
    
    def _take_snapshot_cb(self, gui) :
        """ GUI take snapshot callback"""
        self.debug('RX take-snapshot signal')
        self.project_ctrl.ctrl_acq.do_snapshot()

    def _active_mixer_cb(self, gui, is_active) :
        """ GUI active mixer callback"""
        self.debug('RX active-mixer signal : %s', is_active)
        self.project_ctrl.ctrl_acq.active_mixer(is_active)

    #
    # Timeline callbacks
    # 
    def _change_timeline_order_cb(self, gui, objs, target):
        """ GUI change timeline sequence callback"""
        self.debug(("RX change-timeline-order signal"
                    " - objs %s target %s")
                    , objs, target)
        self.project_ctrl.ctrl_tmln.change_order(objs, target)

    def _insert_image_on_timeline_cb(self, gui, objs, target):
        """ GUI insert image on  timeline callback"""
        self.debug(("RX insert-image-on-timeline signal"
                    "- objs %s target %s"), 
                    objs, target)
        self.project_ctrl.ctrl_tmln.insert(objs, target)

    def _remove_image_on_timeline_cb(self, gui, objs):
        """ GUI remove image on  timeline callback"""
        self.debug('RX remove-image-on-timeline signal - objs %s', objs)
        self.project_ctrl.ctrl_tmln.remove(objs)

    def _active_sound_cb(self, gui) :
        """ GUI active sound callback"""
        self.debug('RX active-sound signal') 
        self.project_ctrl.ctrl_tmln.add_sound_track_to_timeline()

    def _deactive_sound_cb(self, gui) :
        """ GUI deactive sound callback"""
        self.debug('RX deactive-sound signal')
        self.project_ctrl.ctrl_tmln.remove_sound_track_from_timeline()

    def _sound_stop_with_video_cb(self, gui, is_stop) :
        """ GUI sound stop with video callback"""
        self.debug('RX sound-stop-with-video, is_stop %s', is_stop)
        self.project_ctrl.ctrl_tmln.stop_sound_with_video(is_stop) 
    
    def _change_framerate_cb(self, gui, fpi) :
        """ GUI cahnge framerate callback"""
        self.debug('RX change-framerate, fpi %s', fpi)
        self.project_ctrl.set_framerate(fpi)

class CtrlApp(Signallable, Loggable) :
    # CONSTANTS
    MODES = ("INIT", "RUN", "QUIT")

    def __init__(self) :
        """ 
        Controller initialisation.
        Is the main appication intialisation 
        """
        # initialize threads
        gobject.threads_init() 
        
        Loggable.__init__(self)
        
        self.mode = "INIT"
        # init logging as early as possible so we can log startup code
        enable_color = os.environ.get('PITIVI_DEBUG_NO_COLOR', '0') in ('', '0')
        log.init('PITIVI_DEBUG', enable_color)

        self.mainloop = gobject.MainLoop()
        
        # initializes i18n
        self._init_i18n()
        
        # load configuration : ie. saved user options
        self.configuration = LCONF.LucioleConf()
        #  load GUI theme i.e parse rc file
        self.configuration.load_theme()
        
                
        

    def run(self, args) :
        
        # TODO : seems no more needed : check
        import gtk.gdk
        gtk.gdk.threads_enter()
        
        
        # intializes options :
        (_options, _args) = OPTIONS.options_parser(args)
        
        # initialize sound player i.e from snapshot sound
        self.soundplayer = LSOUND.SoundPlayer(LCONST.SOUND_SNAPSHOT_PATH)
        
        # start Project controller
        self.project_ctrl = CTRL_PROJECT.CtrlProject()

        # start GUI
        self.gui = GUI_MAIN.LucioleMainWindow(self)
        

        # connect project controller with GUI
        self.project_ctrl.connect_to_gui(self.gui)
        
        self.connect_to_project(self.project_ctrl)       
        
        self.gui_interface = CtrlAppGui(self)

        # load a project ?
        if _options.filename \
            and os.path.exists(_options.filename):
            self.project_ctrl.open(_options.filename)
        else :
            self.project_ctrl.no_project_mode()
        self.mode = "RUN"

        

        self.mainloop.run()

        # TODO : seems no more needed : check
        import gtk.gdk
        gtk.gdk.threads_leave()
    
    def quit(self) :
        self.mainloop.quit()

    def _init_i18n(self) :
        """ intialize locales """
        _locale_path = LCONST.LOCALE_DIR

        # Init the list of languages to support
        _langs = []
        #Check the default locale
        _lc, _encoding = locale.getdefaultlocale()
        if (_lc):
            #If we have a default, it's the first in the list
            _langs = [_lc]
        # Now lets get all of the supported languages on the system
        _language = os.environ.get('LANGUAGE', None)
        _msg = "language : " , _language
        self.debug(_msg)
        if (_language):
            # langage comes back something like en_CA:en_US:en_GB:en
            # on linuxy systems, on Win32 it's nothing, so we need to
            # split it up into a list
            _langs += _language.split(":")
        # Now add on to the back of the list the translations that we
        # know that we have, our defaults"""
        _langs += ["en_US"]
        
        # Now langs is a list of all of the languages that we are going
        # to try to use.  First we check the default, then what the system
        # told us, and finally the 'known' list
        _msg = "locale path", _locale_path
        self.debug(_msg)
        
        GUI_CTRL.GuiController.init_18n(_locale_path)
        #import gtk.glade
        #gtk.glade.bindtextdomain(LCONST.APP_NAME, _locale_path)
        #gtk.glade.textdomain(LCONST.APP_NAME)

        gettext.bindtextdomain(LCONST.APP_NAME, _locale_path)
        gettext.textdomain(LCONST.APP_NAME)
        
        # Get the language to use
        _lang = gettext.translation(LCONST.APP_NAME, _locale_path
            , languages=_langs, fallback = True)
        # Install the language, map _() (which we marked our
        # strings to translate with) to self.lang.gettext() which will
        # translate them."""
        global _
        _ = _lang.gettext

    def connect_to_project(self, project_ctrl) :
        project_ctrl.connect('project-loaded', self._project_loaded_cb)
        project_ctrl.connect('project-closed', self._project_closed_cb)
        project_ctrl.connect('quit', self._quit_cb)

    def disconnect_from_project(self, project_ctrl) :
        project_ctrl.disconnect_by_function(self._project_loaded_cb)
        project_ctrl.disconnect_by_function(self._project_closed_cb)
    
    def _project_loaded_cb(self, project_ctrl, project) :
        self.soundplayer.connect_to_app(project_ctrl)

    def _project_closed_cb(self, project_ctrl) :
        pass
        #self.soundplayer.disconnect_from_app(project_ctrl)
    
    def _quit_cb(self, project) :
        self.quit()



   


