#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    webcam detetction widget
"""


# standard library imports
from gettext import gettext as _

# related third party imports

import gtk
import gobject


from pitivi.log.loggable import Loggable
from pitivi.signalinterface import Signallable

# for test
from pitivi.pipeline import Pipeline

from pitivi.action import ViewAction
# local application/library specific imports
import luciole.media.webcam_detect.webcam_detection as WBCM_DTCT

# for test
from luciole.media.lgst.webcam_factory import WebcamSourceFactory

class WebcamField(Loggable, Signallable) :
    """ Manage webcam properties/fields """
    __signals__ = {
                'combo-changed' : ['field','value'],
        }

    def __init__(self, field_name, value, widget_type = 'combo' ) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.field_name = field_name
        self.value = value
        self.widget_type = widget_type
        self._combo_idx = []
        self._combo = None
        self._range = None
        self.widget = self._create_ui()

    def _create_ui(self) :
        """ Gui widgetc creation """
        _bbox =  gtk.HBox()
        _boxalign = gtk.Alignment(xalign=0.0, yalign=0.0)
        _boxalign.add(_bbox)
        
        _label = gtk.Label(str(self.field_name))
        _bbox.pack_start(_label, expand=False)

        if isinstance(self.value, list) :
            if self.widget_type == 'combo' :
                self._combo = self._create_combo(_bbox)
            elif self.widget_type == 'range':
                self._range = self._create_range(_bbox)
            else :
                self.error('Unknow widget type %s', self.widget_type)
        else :
            _label = gtk.Label(str(self.value))
            _bbox.pack_start(_label, expand=False)
        return _boxalign
    
    def _create_combo(self, bbox) :
        """ create GUI combo box """
        _combo = gtk.combo_box_new_text()
        self.debug('create combobox for field %s', self.field_name)
        for _idx, _value in enumerate(self.value) :
            _combo.append_text(str(_value))
            self._combo_idx.append(str(_value))
        _combo.connect('changed', self._combobox_defintion_changed_cb)
        bbox.pack_start(_combo, expand=False)
        return _combo
    
    def _create_range(self, bbox) :
        """ create GUI range """
        
        _adj = gtk.Adjustment(
                float(self.value[0]),
                float(self.value[0]),
                float(self.value[1]),
                0.1,
                1.0,
                1.0)
        _scale = gtk.HScale(_adj)
        _scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
        _scale.set_digits(0)
        _scale.set_value_pos(gtk.POS_RIGHT)
        _scale.set_draw_value(True)
        _scale.set_size_request(500, 500)
        _scale.connect('value-changed', self._vscale_value_changed_cb)
        bbox.pack_start(_scale, expand=True, fill = True )
        return _scale

    def _combobox_defintion_changed_cb(self, widget) :
        """ combobox change callback"""
        self.emit('combo-changed', self.field_name, widget.get_active_text() )
    
    def _vscale_value_changed_cb(self, widget) :
        """ scale value changed callback"""
        self.emit('combo-changed', self.field_name, widget.get_value() )


    def set_active(self, value) :
        """ active values on combo """
        if self._combo is not None :
            try :
                _idx = self._combo_idx.index(value)
            except :
                self.error('value %s not found in combo', value)
            else :
                self._combo.set_active(_idx)


class WebcamPropsWdg(Loggable, Signallable) :
    """ Webcam Properties widget """ 
   
    __signals__ = {
        'prop-changed' : ['prop','value'],
        'webcam-changed' : ['webcam'],
        }

    _MAIN_WDG = "vbox_main"
    

    def __init__(self, parent_wdg) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        
        self.prop_widgets = {}
        self.webcam_name = None
        self._main_window = gtk.VBox()

        parent_wdg.add(self._main_window)
        
        
        # Show the window and all its children
        self._main_window.show_all()

           
  
    def set_definitions(self, definitions) :
        """ set webcam definitions GUI prop"""
        self.set_prop('definition', definitions) 
    
    def set_active_defintion(self, definition) :
        """ set webcam active definition GUI prop"""
        _wdg = self.prop_widgets['definition']
        _wdg.set_active(definition)


    def set_videotypes(self, videotypes) :
        """ set webcam videotypes GUI prop"""
        self.set_prop('videotype', videotypes) 

    def set_active_videotype(self, value) :
        """ set webcam active videotype GUI prop"""
        self.set_active_prop('videotype', value) 
    
    
    def set_prop(self, prop, prop_values, wdg_type='combo' ) :
        """ set a webcam GUI prop. Generic method """
        _wdg = WebcamField(prop, prop_values, wdg_type )
        if isinstance(prop_values, list) :
            _wdg.connect('combo-changed', self._combo_changed_cb)
            self.prop_widgets[prop] = _wdg
        self._main_window.pack_start(_wdg.widget) 
        _wdg.widget.show_all()

    
    def set_active_prop(self, prop, active_value):
        """ set a webcam active GUI prop. Generic method """
        _wdg = self.prop_widgets[prop]
        _wdg.set_active(active_value)
    
    def remove_prop(self, prop) :
        """ remove a webcam property """
        if prop in self.prop_widgets :
        
            self.debug('remove prop %s', prop)
            _wdg =  self.prop_widgets[prop]
        
            self._main_window.remove(_wdg.widget)
            del self.prop_widgets[prop]
    
    def remove_all_props(self) :
        """ remove all properties """
        _props =  self.prop_widgets.keys()
        for _prop in _props:
            _wdg = self.prop_widgets.pop(_prop)
            self._main_window.remove(_wdg.widget)
            _wdg.disconnect_by_function(self._combo_changed_cb)
            self.debug("remove widget prop %s", _prop)


    def _combo_changed_cb(self, widget, prop, value) :
        """ combobox value changed callback """
        self.debug('emit prop-changed : %s=%s', prop, value)
        self.emit('prop-changed', prop, value)
    

    def display_webcams(self, _webcams) :
        """ display detected webcams """
        _wdg = WebcamField('webcam-name', _webcams)
        if isinstance(_webcams, list) :
            _wdg.connect('combo-changed', self._webcam_changed_cb)
            self.webcam_name = _wdg

        self._main_window.pack_start(_wdg.widget) 
        _wdg.widget.show_all()
    
    def _webcam_changed_cb(self, widget, prop, value) :
        """ webcam selection callback """
        self.debug('emit webcam-changed : %s=%s', prop, value)
        
        self.emit('webcam-changed', value)


class  WebcamDetectionWidget(gtk.Frame, Loggable) :
    """ Assistant page for webcam detection """

    __gsignals__ = { 
           'webcams-detected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                               (gobject.TYPE_PYOBJECT,)),
           'webcam-not-detected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
               (gobject.TYPE_PYOBJECT,) ),
           'webcam-selected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
               (gobject.TYPE_PYOBJECT,) ),
           'webcam-caps-selected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                               (gobject.TYPE_PYOBJECT,))

           
           }


        
    def __init__(self, project_data, viewer, *args ) :
        gtk.Frame.__init__(self, *args )
        Loggable.__init__(self)
        self.wcam_params = {}
        self._viewer = viewer
        self.factory = None
        self.action = None
        self.pipeline = None
        self.set_name('Page_webcam')
        self.project_data = project_data
        
        #
        # set default values
        #
        self.project_data['webcam_data'] = dict()
        
        #
        # Initailize widgets for webcam detection page 
        #
        self.webcam_props_widget = WebcamPropsWdg(self)
        
        self.webcam_props = WBCM_DTCT.WebcamProps(self.webcam_props_widget) 
        self.webcam_props.connect('caps-selected', self._caps_selected_cb)
        
        self._wcam_detect =  WBCM_DTCT.WebcamDetection()
        self.debug('Initialized')

        self.show_all()
        self.pipeline = None

    def prepare_webcam_detection(self) :
        """ prepare assistant page for the webcam detection page """
        
        # launch detection
        _nb = self._wcam_detect.detect_webcams()

        if _nb > 0 :
            _webcams = [ _device['name'] for _device in \
                    self._wcam_detect.webcam_devices]
            self.webcam_props_widget.display_webcams(_webcams)
            self.emit('webcams-detected', _webcams)
            self.webcam_props_widget.connect(
                    'webcam-changed', 
                    self._webcam_changed_cb) 
        else :
            self.info('No webcam detected')
            self.emit('webcam-not-detetcted')

    def prepare_viewer(self, wcam_params) :
        """ prepare viewer and pipeline """
        self.debug('prepare pipeline/viewer start')

        viewer = self._viewer
        if viewer.pipeline is not None :
            try :
                viewer.pause()
            except :
                self.warning('Impossible to pause viewer')
        self.factory = WebcamSourceFactory()
        self.factory.webcam_params =  wcam_params
        self.factory.with_scale = False

        self.pipeline = Pipeline()
        self.action = ViewAction()
        self.action.addProducers(self.factory)
        viewer.setPipeline(None)
        #viewer.showSlider()
        
        # FIXME: why do I have to call viewer.setAction ?
        viewer.setAction(self.action)
        viewer.setPipeline(self.pipeline)
        viewer.play()
        self.debug('viewer started')
 
    def release_viewer(self) :
        """ release pipeline for viewer"""
        self.debug(' release pipe %s', self.pipeline)

        viewer = self._viewer
        if self.pipeline is None  :
            self.debug('No pipeline to release')
            return
        if viewer.pipeline is not None :
            try :
                viewer.stop()
            except :
                self.warning('Impossible to stop pipeline')

        viewer.setPipeline(None)
        self.pipeline.release()
        self.pipeline = None
        self.factory = None
        self.action = None


    def _webcam_changed_cb(self, wdg, webcam) :
        """ webcam selection changed callback """
        self.debug('Rx webcam-changed : %s', webcam )
        self.webcam_props.clear_all()
        self.wcam_params = {}
        _selected_device = None
        
        for _device in self._wcam_detect.webcam_devices :
            if _device['name'] == webcam :
                _selected_device = _device
                break
        if _selected_device != None :
            # Emit webcam detetected
            self.debug('Emit webcam-selected device : %s ', _selected_device)
            self.project_data['webcam_data'] = _selected_device
            self.emit('webcam-selected', _selected_device)
            
            # prepare webcam param/caps detection
            self.wcam_params['wcam_driver'] = {
                    'element': _selected_device['v4ldriver'] ,
                    'properties' : {
                        'device':_selected_device['device-file'],
                        }
                    }
            self.prepare_viewer(self.wcam_params)
            self.webcam_props.prepare(self.factory.webcam_caps)


        else :
            self.error('No selected device. Unknow %s', webcam)
    

    def _caps_selected_cb(self, webcam_props, caps) :
        """ webcam caps(propeties) selected callback """
        self.project_data['webcam_data']['caps'] = caps
        
        self.wcam_params['wcam_filter'] = {
                'media_type' : caps,
                }
        
        self.prepare_viewer(self.wcam_params)

        self.debug('Rx caps-selected : %s', caps.to_string() )
        self.emit('webcam-caps-selected', caps)
        
    

